// TAB_SIZE = 4
var AudioContext = window.AudioContext || window.webkitAudioContext;
var MediaStream = window.MediaStream || window.webkitMediaStream;
var MediaRecorder = window.MediaRecorder || window.webkitMediaRecorder;

window.addEventListener('load', function() {
	var requestAnimationFrame =
		window.requestAnimationFrame ||
		window.mozRequestAnimationFrame ||
		window.webkitRequestAnimationFrame ||
		window.oRequestAnimationFrame ||
		function(callback){ setTimeout(callback, 1000/60); };
	
	var cvs = document.getElementById("cvs");
	var gtx = cvs.getContext('2d');
	var ctx = new AudioContext();
	
	var audioSource;
	var analyser;
	var freqData;
	
	var audioElement = document.getElementById("audioElement");
	var buttonRecord = document.getElementById("videoRecord");
	
	var cvs_strm_track = cvs.captureStream().getTracks()[0];
	var aud_strm_track = ctx.createMediaStreamDestination().stream.getTracks()[0];
	var recorder = new MediaRecorder(new MediaStream([cvs_strm_track, aud_strm_track]));
	
	function processAudioDataURL(title, theurl) {
		audioElement.src = theurl;
		audioElement.play();
	}
	
	function processAudioFile(soundFile) {
		if(!soundFile.type.match('audio.*')) {
			return;
		}
		
		var reader = new FileReader();
		reader.addEventListener('load', function(e) {
			processAudioDataURL(soundFile.name.substr(0, soundFile.name.lastIndexOf('.')), e.target.result);
		});
		
		reader.readAsDataURL(soundFile);
	}
	
	function loop() {
		if(!freqData) {
			freqData = new Float32Array(analyser.frequencyBinCount);
			
			if(audioElement.paused) {
				for(var i = 0; i < freqData.length; i++) {
					freqData[i] = Number.MIN_SAFE_INTEGER;
				}
			}
		}
		
		if(!audioElement.paused) {
			analyser.getFloatFrequencyData(freqData);
		}
		
		if(cvs.width != cvs.clientWidth || cvs.height != cvs.clientHeight) {
			cvs.width = cvs.clientWidth;
			cvs.height = cvs.clientHeight;
		}
		
		render();
		
		requestAnimationFrame(loop);
	}
	
	function render() {
		gtx.clearRect(0, 0, cvs.width, cvs.height);
		gtx.strokeStyle = "white";
		gtx.lineWidth = 4;
		
		gtx.beginPath();
		for(var i = 0, l = freqData.length; i < l; i++) {
			var freqValue = Math.max(Math.min(Math.max(freqData[i] + 100, 0.0) / 60, 1.0) * cvs.height / 4, 2);
			
			gtx.moveTo(i / l * cvs.width, cvs.height / 2 + freqValue);
			gtx.lineTo(i / l * cvs.width, cvs.height / 2 - freqValue);
		}
		gtx.stroke();
	}
	
	if(!cvs || !gtx || !ctx) {
		alert("Your browser isn't compatible"); 
		throw new Error("Couldn't initialize");
		
		return;
	}
	
	audioSource = ctx.createMediaElementSource(audioElement);
	
	analyser = ctx.createAnalyser();
	analyser.fftSize = 512;
	analyser.smoothingTimeConstant = 0.5;
	
	cvs.width = document.body.clientWidth;
	cvs.height = document.body.clientHeight;
	
	cvs.addEventListener('dragover', function(e) {
		e.stopPropagation();
		e.preventDefault();
		
		e.dataTransfer.dropEffect = 'copy';
	}, false);
	
	cvs.addEventListener('drop', function(e) {
		e.stopPropagation();
		e.preventDefault();
		
		processAudioFile(e.dataTransfer.files[0]);
	}, false);
	
	audioElement.crossOrigin = "anonymous";
	
	var allButtons = document.getElementsByClassName('button');
	
	var recordingChunks = [];
	recorder.addEventListener('stop', function(e) {
		var blob = new Blob(recordingChunks, {'type': 'video/webm'});
		recordingChunks = [];
		
		downloader.href = URL.createObjectURL(blob);
		downloader.download = "recording.webm";
		
		downloader.click();
		
		buttonRecord.innerHTML = "Record a video";
		buttonRecord.classList.remove('disabled');
	});
	recorder.addEventListener('dataavailable', function(e) {
		recordingChunks.push(e.data);
	});
	
	buttonRecord.addEventListener('click', function() {
		if(this.classList.contains('disabled')) return;
		
		if(recorder.state === 'recording') {
			recorder.stop();
			
			return;
		}
		
		// Start recording
		audioElement.currentTime = 0;
		recorder.start();
		
		this.innerHTML = "Stop the record";
	});
	
	for(var i = 0; i < allButtons.length; i++) {
		var b = allButtons[i];
		
		b.addEventListener('mousedown', function() {
			this.classList.add('pushed');
		});
		b.addEventListener('mouseup', function() {
			this.classList.remove('pushed');
		});
	}
	
	audioElement.addEventListener('ended', function() {
		if(recorder.state === 'recording') recorder.stop();
	});
	audioElement.addEventListener('pause', function() {
		if(recorder.state === 'recording') recorder.pause();
	});
	audioElement.addEventListener('play', function() {
		if(recorder.state === 'paused') recorder.resume();
	});
	
	audioSource.connect(analyser);
	analyser.connect(ctx.destination);
	
	loop();
});
